@@ -27,7 +27,7 @@ from tiddlyweb.store import (NoBagError, NoRecipeError, NoTiddlerError,
 from tiddlyweb.stores import StorageInterface
 from tiddlyweb.util import binary_tiddler
 
-__version__ = '3.0.1'
+__version__ = '3.0.2'
 
 #logging.basicConfig()
 #logging.getLogger('sqlalchemy.engine').setLevel(logging.INFO)
@@ -525,7 +525,8 @@ class Store(StorageInterface):
                 sbag = self.session.query(sBag.id).filter(sBag.name
                         == tiddler.bag).one()
             except NoResultFound, exc:
-                raise NoBagError('bag must exist for tiddler save: %s', exc)
+                raise NoBagError('bag %s must exist for tiddler save: %s'
+                        % (tiddler.bag, exc))
             current_revision_number = self._store_tiddler(tiddler)
             tiddler.revision = current_revision_number
             self.session.commit()
@@ -743,7 +744,7 @@ class Store(StorageInterface):
         text.revision_number = srevision.number
         self.session.add(text)
 
-        for tag in tiddler.tags:
+        for tag in set(tiddler.tags):
             stag = sTag(tag)
             stag.revision_number = srevision.number
             self.session.add(stag)
